/*
  XString - string class for C++
  Copyright Keristor Systems and Chris Croughton 1997
  Internet: chris@keris.demon.co.uk
  
  This software is free software; you can redistribute it and/or modify
  it freely under the following conditions:

  1) The copyright of this software belongs to the above copyright holders,
     and all versions of the software must carry (at least in source form)
     the copyright notice and these conditions.

  2) Permission is granted for any non-commercial use for any purpose; if
     the software is distributed in modified form then in addition to the
     previous restriction it must also bear a notice stating that it has
     been modified and by whom.

  3) Any modifications are your responsibility; if you wish to pass them on
     to the authors then we reserve the right to include them or not, as we
     decide, in future versions of the software.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

// NOTE:
// The interfaces here are subject to change, this software is still
// under development.  In particular, the '+' operators for concatenation
// of strings are being replaced by equivalent '|' operators; the '+' 
// operators will eventually have a different effect. 

// $Id: xstring.cc,v 1.10 1998/01/14 12:40:54 crough45 Exp crough45 $
//
// $Log: xstring.cc,v $
// Revision 1.10  1998/01/14  12:40:54  crough45
// Changed header.
//
// Revision 1.9  1997/11/10  13:35:34  crough45
// Convert return type of Find* functions to long rather than
// int, to allow maximum string length plus -1 for not found.
//
// Revision 1.8  1997/11/10  13:20:12  crough45
// Usage condition changes
// Add constructors taking integers (to be converted to ASCII)
// Add MixedCase conversion functions
// Add PadRight, PadLeft and Centre functions
// Add start character number to Find* functions
//
// Revision 1.7  1997/07/21  12:14:40  crough45
// Added traces for integral concatenation functions and
// test to the 'main' function
//
// Revision 1.6  1997/07/21  11:43:42  crough45
// Added operators | etc. to replace + for concatenation;
// added concatenation of integer values to append the
// values in decimal
//
// Revision 1.5  1997/03/21  11:04:16  crough45
// *** empty log message ***
//
// Revision 1.2  1997/02/21  17:56:02  crough45
// Changed header
//
// Revision 1.1  1997/02/18  12:42:51  crough45
// Initial revision
//
// Revision 1.1  1997/02/18  12:42:51  crough45
// Initial revision

#include <stddef.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "xstring.h"

#ifdef TRACE_XSTRING
# define XXDO_TRACE
# include "trace.h"
#elif defined __GNUC__
# define TRACE(s,a...)
#else
# define TRACE
#endif

//////////////////////////////////////////////////////////////////////////

#define STR(s) ((s)?(s):"")

//////////////////////////////////////////////////////////////////////////

#ifdef __STDC__

#define throw(x) \
do{\
      printf("%s.%d {%s} *** THROW: %s ***\n", \
             __FILE__, __LINE__, __PRETTY_FUNCTION__, x);\
             exit(99);}while(0)

static int
_XString_stricmp(const char * a, const char * b)
{
    if (!a)
        return (b ? -1 : 0);
    if (!b)
        return 1;
    while (*a && *b)
    {
        if (tolower(*a) != tolower(*b))
            return (tolower(*a) > tolower(*b) ? 1 : -1);
        a++;
        b++;
    }
    if (tolower(*a) == tolower(*b))
        return 0;
    return (tolower(*a) > tolower(*b) ? 1 : -1);
}

#define stricmp(a,b)    _XString_stricmp(a,b)

static char *
_Xstrdup(const char *s)
{
   if (!s)
      s = "";
   char *p = (char*)malloc(strlen(s)+1);
   if (!p)
   {
      fprintf(stderr, "*** no memory for strdup(<%s>)\n", s);
      exit(1);
   }
   strcpy(p, s);
//   TRACE("%p '%s' -> %p '%s'", s, s, p, p);
   return p;
}

#define strdup(s) _Xstrdup(s)

#endif

//////////////////////////////////////////////////////////////////////////

XString::XString()
{
    m_string = NULL;
}

XString::XString(const XString& string)
{
    m_string = strdup((const char *)string);
    if (!m_string)
        throw("XString: no memory");
    TRACE("= %p \"%s\"", m_string, m_string);
}

XString::XString(char ch, int nRepeat)
{
    m_string = (char*)malloc(nRepeat+1);
    if (!m_string)
        throw("XString: no memory");
    memset(m_string, ch, nRepeat);
    m_string[nRepeat] = 0;
    TRACE("= %p \"%s\"", m_string, m_string);
}

XString::XString(const char * lpsz)
{
    m_string = strdup(lpsz);
    if (!m_string)
        throw("XString: no memory");
    TRACE("= %p \"%s\"", m_string, m_string);
}

XString::XString(const char * lpch, int nLength)
{
   if (!lpch)
      lpch = "";
   int len = strlen(lpch);
   if (nLength > len)
      nLength = len;
   m_string = (char*)malloc(nLength+1);
   if (!m_string)
      throw("XString: no memory");
   strncpy(m_string, lpch, nLength);
   m_string[nLength] = 0;
   TRACE("= %p \"%s\"", m_string, m_string);
}

XString::XString(int n)
{
  m_string = (char*)malloc(sizeof(long)*4+2);
  TRACE("= %ld", (long)n);
  sprintf(m_string, "%ld", (long)n);
}

XString::XString(unsigned int n)
{
  m_string = (char*)malloc(sizeof(long)*4+2);
  TRACE("= %lu", (long)n);
  sprintf(m_string, "%lu", (long)n);
}

XString::XString(long n)
{
  m_string = (char*)malloc(sizeof(long)*4+2);
  TRACE("= %ld", (long)n);
  sprintf(m_string, "%ld", (long)n);
}

XString::XString(unsigned long n)
{
  m_string = (char*)malloc(sizeof(long)*4+2);
  TRACE("= %lu", (long)n);
  sprintf(m_string, "%lu", (long)n);
}

XString::~XString()
{
   if (m_string)
   {
      TRACE("~ %p \"%s\"", m_string, m_string);
      free(m_string);
   }
}


// Attributes & Operations
	// as an array of characters
int XString::Len() const
{
    return (m_string ? strlen(m_string) : 0);
}

char XString::IsEmpty() const
{
    return (m_string ? *m_string == 0 : 1);
}

void XString::Empty()                       
{
    if (m_string)
    {
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
    }
    m_string = NULL;
}

char XString::GetAt(int nIndex) const      
{
    return (m_string && nIndex >= 0 && (size_t)nIndex < strlen(m_string) 
            ? m_string[nIndex] : 0);
}

char XString::operator[](int nIndex) const 
{
    return (m_string && nIndex >= 0 && (size_t)nIndex < strlen(m_string) 
            ? m_string[nIndex] : 0);
}

void XString::SetAt(int nIndex, char ch)
{
    if (m_string && nIndex >= 0 && (size_t)nIndex < strlen(m_string))
        m_string[nIndex] = ch;
}

//XString::operator const char *() const         
//{
//    return (m_string ? m_string : "");
//}


// overloaded assignment
const XString& XString::operator=(const XString& str)
{
    if (m_string)
    {
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
    }
    m_string = (str.m_string ? strdup(str.m_string) : (char*)NULL);
    TRACE("= %p \"%s\"", m_string, m_string ? m_string : "");
    return *this;
}

const XString& XString::operator=(char ch)
{
    m_string = (char*)malloc(2);
    if (!m_string)
        throw("XString: no memory");
    m_string[0] = ch;
    m_string[1] = 0;
    return *this;
}

const XString& XString::operator=(const char * lpsz)
{
    if (m_string)
    {
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
    }
    m_string = (lpsz ? strdup(lpsz) : (char*)NULL);
    return *this;
}

const XString& XString::operator=(const unsigned char* psz)
{
    if (m_string)
    {
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
    }
    m_string = (psz ? strdup((const char *)psz) : (char*)NULL);
    return *this;
}


const XString& XString::operator+=(const XString& string)
{
    const char *s = (const char *)string;
    TRACE("+= (%p \"%s\", %p \"%s\")", 
          m_string, STR(m_string), s, STR(s));
    if (m_string)
    {
        int len = strlen(m_string);
        char *p = (char*)malloc(len+strlen(s)+1);
        if (!p)
            throw("XString: no memory");
        strcpy(p, m_string);
        strcpy(p+len, s);
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
        m_string = p;
    }
    else
    {
        *this = string;
    }
    return *this;
}

const XString& XString::operator+=(char ch)
{
    TRACE("+= (%p \"%s\", '%c')", 
          m_string, STR(m_string), ch);
    if (m_string)
    {
        int len = strlen(m_string)+1;
        char *p = (char*)malloc(len+1);
        if (!p)
            throw("XString: no memory");
        strcpy(p, m_string);
        p[len-1] = ch;
        p[len] = 0;
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
        m_string = p;
    }
    else
    { 
        *this = ch;
    }
    return *this;
}

const XString& XString::operator+=(const char * lpsz)
{
    if (!lpsz)
        return *this;
    TRACE("+= (%p \"%s\", %p \"%s\")", 
          m_string, STR(m_string), lpsz, STR(lpsz));
    if (m_string)
    {
        int len = strlen(m_string)+strlen(lpsz);
        char *p = (char*)malloc(len+1);
        if (!p)
            throw("XString: no memory");
        strcpy(p, m_string);
        strcat(p, lpsz);
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
        m_string = p;
    }
    else
    {
        *this = lpsz;
    }
    return *this;
}


XString operator+(const XString& string1, const XString& string2)
{
    const char *s1 = (const char *)string1;
    const char *s2 = (const char *)string2;
    XString s;
    int len = strlen(s1);
    TRACE("+ (%p \"%s\", %p \"%s\")", 
          s1, STR(s1), s2, STR(s2));
    s.m_string = (char*)malloc(len+strlen(s2)+1);
    if (!s.m_string)
        throw("XString: no memory");
    strcpy(s.m_string, s1);
    strcpy(s.m_string+len, s2);
    TRACE("= %p \"%s\"", s.m_string, s.m_string ? s.m_string : "");
    return s;
}

XString operator+(const XString& string1, char ch)
{
    const char *s1 = (const char *)string1;
    XString s;
    int     len = strlen(s1)+1;
    TRACE("+ (%p \"%s\", '%c')", 
          s1, STR(s1), ch);
    s.m_string = (char*)malloc(len+1);
    if (!s.m_string)
        throw("XString: no memory");
    strcpy(s.m_string, s1);
    s.m_string[len-1] = ch;
    s.m_string[len] = 0;
    TRACE("= %p \"%s\"", s.m_string, s.m_string ? s.m_string : "");
    return s;
}

XString operator+(char ch, const XString& string1)
{
    const char *s1 = (const char *)string1;
    XString s;
    int     len = strlen(s1)+1;
    TRACE("+ ('%c', %p \"%s\")", 
          ch, s1, STR(s1));
    s.m_string = (char*)malloc(len+1);
    if (!s.m_string)
        throw("XString: no memory");
    s.m_string[0] = ch;
    strcpy(s.m_string+1, s1);
    TRACE("= %p \"%s\"", s.m_string, s.m_string ? s.m_string : "");
    return s;
}

XString operator+(const XString& string1, const char * lpsz)
{
    if (!lpsz)
       return string1;
    const char *s1 = (const char *)string1;
    XString s;
    int     len = strlen(s1);
    TRACE("+ (%p \"%s\", %p \"%s\")", 
          s1, STR(s1), lpsz, STR(lpsz));
    s.m_string = (char*)malloc(len+strlen(lpsz)+1);
    if (!s.m_string)
        throw("XString: no memory");
    strcpy(s.m_string, s1);
    strcpy(s.m_string+len, lpsz);
    TRACE("= %p \"%s\"", s.m_string, s.m_string ? s.m_string : "");
    return s;
}

XString operator+(const char * lpsz, const XString& string1)
{
    if (!lpsz)
       return string1;
    TRACE("+ (%p \"%s\", %p \"%s\")", 
          lpsz, STR(lpsz), (const char *)string1, STR((const char *)string1));
    if (!string1.m_string)
       return (XString)lpsz;
    const char *s1 = (const char *)string1;
    XString s;
    int     len = strlen(lpsz);
    s.m_string = (char*)malloc(len+strlen(s1)+1);
    if (!s.m_string)
        throw("XString: no memory");
    strcpy(s.m_string, lpsz);
    strcpy(s.m_string+len, s1);
    TRACE("= %p \"%s\"", s.m_string, s.m_string ? s.m_string : "");
    return s;
}


XString operator|(const XString& s, long n)
{
  char buff[sizeof(long)*4+2];
  TRACE("| (%p \"%s\",  %ld)", (const char *)s, STR((const char *)s), n);
  sprintf(buff, "%ld", n);
  return s | buff;
}

XString operator|(const XString& s, unsigned long n)
{
  char buff[sizeof(long)*4+2];
  TRACE("| (%p \"%s\",  %ld)", (const char *)s, STR((const char *)s), n);
  sprintf(buff, "%lu", n);
  return s | buff;
}
      

	// string comparison
int XString::Compare(const char * lpsz) const
{
    if (m_string)
        return (lpsz ? strcmp(m_string, lpsz) : 1);
    return (lpsz ? -1 : 0);
}

int XString::CompareNoCase(const char * lpsz) const
{
    if (m_string)
        return (lpsz ? stricmp(m_string, lpsz) : 1);
    return (lpsz ? -1 : 0);
}

int XString::Collate(const char * lpsz) const
{
    if (m_string)
        return (lpsz ? strcmp(m_string, lpsz) : 1);
    return (lpsz ? -1 : 0);
}


XString XString::Mid(int nFirst, int nCount) const
{
    XString s = Mid(nFirst).Left(nCount);
    return s;
}

XString XString::Mid(int nFirst) const
{
    XString s;
    if (m_string && nFirst < (int)strlen(m_string))
    {
        if (nFirst < 0)
            nFirst = 0;
        s = m_string + nFirst;
    }
    return s;
}

XString XString::Left(int nCount) const
{
    XString s(m_string, nCount);
    return s;
}

const XString& XString::operator<<=(const int nChars)
{
    if (m_string)
    {
        int len = strlen(m_string);
        if (len > nChars)
        {
            char *p = (char*)malloc(len+1 - nChars);
            if (!p)
                throw("XString: no memory");
            strcpy(p, m_string+nChars);
            TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
            free(m_string);
            m_string = p;
        }
        else
        {
            TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
            free(m_string);
            m_string = NULL;
        }
    }
    return *this;
}

XString XString::Right(int nCount) const
{
    if (!m_string)
       return "";
    int len = Len();
    if (nCount > 0)
        len = (nCount < len ? len - nCount : 0);
    XString s(m_string + len);
    return s;
}


XString XString::SpanIncluding(const char * lpszCharSet) const
{
    char *p = m_string;
    while (strchr(lpszCharSet, *p))
      p++;
    return Left(p-m_string);
}

XString XString::SpanExcluding(const char * lpszCharSet) const
{
    char *p = m_string;
    while (!strchr(lpszCharSet, *p))
      p++;
    return Left(p-m_string);
}


void XString::ToUpper()
{
    char *p = m_string;
    while (p && *p)
    {
        *p = toupper(*p);
        p++;
    }
}

void XString::ToLower()
{
    char *p = m_string;
    while (p && *p)
    {
        *p = tolower(*p);
        p++;
    }
}

void XString::ToOtherCase()
{
    char *p = m_string;
    while (p && *p)
    {
        *p = (islower(*p) ? toupper(*p) : tolower(*p));
        p++;
    }
}

void XString::ToMixedCase()
{
    char *p = m_string;
    while (p && *p)
    {
	if (isalpha(*p) && (p == m_string || isspace(*(p-1))))
	    *p = toupper(*p);
	else
	    *p = tolower(*p);
        p++;
    }
}

XString XString::Upper() const
{
    XString s(*this);
    s.ToUpper();
    return s;
}

XString XString::Lower() const
{
    XString s(*this);
    s.ToLower();
    return s;
}

XString XString::OtherCase() const
{
    XString s(*this);
    s.ToOtherCase();
    return s;
}


XString XString::MixedCase() const
{
    XString s(*this);
    s.ToMixedCase();
    return s;
}


void XString::TrimRight()
{
    if (!m_string)
        return;
    int i = strlen(m_string);
    while (i > 0)
    {
        --i;
        if (isspace(m_string[i]))
            m_string[i] = 0;
        else
            return;
    }
}

void XString::TrimLeft()
{
    if (!m_string)
        return;
    char *p = m_string;
    while (*p && isspace(*p))
        p++;
    if (p != m_string)
    {
        char *q = strdup(p);
        if (!q)
            throw("XString: no memory");
        TRACE("~ %p \"%s\"", m_string, m_string ? m_string : "");
        free(m_string);
        m_string = q;
    }
}

void XString::Trim()
{
    if (!m_string)
        return;
    TrimRight();
    TrimLeft();
}

XString PadRight(const char *s, int len)
{
  XString r(' ', len);
  int ln = strlen(s);
  if (ln > len)
    ln = len;
  strncpy(r.m_string, s, ln);
  return r;
}

XString PadLeft(const char *s, int len)
{
  XString r(' ', len);
  int ln = strlen(s);
  if (ln > len)
    ln = len;
  strncpy(r.m_string+len-ln, s, ln);
  return r;
}

XString Centre(const char *s, int len)
{
  XString r(' ', len);
  int ln = strlen(s);
  if (ln > len)
    ln = len;
  strncpy(r.m_string+(len-ln)/2, s, ln);
  return r;
}



long XString::Find(char ch, size_t first) const               
{
    if (!m_string || strlen(m_string) <= first)
        return -1L;
    char *p = strchr(m_string+first, ch);
    return (p ? p-m_string : -1);
}

long XString::FindLast(char ch, size_t first) const
{
    if (!m_string || strlen(m_string) <= first)
        return -1L;
    char *p = strrchr(m_string+first, ch);
    return (p ? p-m_string : -1);
}

long XString::FindOneOf(const char * lpszCharSet, size_t first) const
{
    if (!m_string || !lpszCharSet || strlen(m_string) <= first)
        return -1L;
    size_t n = strcspn(m_string+first, lpszCharSet);
    return (n < strlen(m_string) ? (int)n : -1);
}


long XString::Find(const char * lpsz, size_t first) const
{
    if (!m_string || !lpsz || strlen(m_string) <= first)
        return -1L;
    char *p = strstr(m_string+first, lpsz);
    return (p ? p-m_string : -1);
}


void XString::Format(const char * lpszFmt, ...)
{
    char *p = new char[8192];
    if (!p)
        throw("XString: no memory");
    va_list ap;
    va_start(ap, lpszFmt);
    vsprintf(p, lpszFmt, ap);
    va_end(ap);
    *this = p;
    delete p;
}

void XString::vFormat(const char * lpszFmt, va_list ap)
{
    char *p = new char[8192];
    if (!p)
        throw("XString: no memory");
    vsprintf(p, lpszFmt, ap);
    *this = p;
    delete p;
}

const char * XString::GetBuffer(size_t nMinBufLength)
{
  if (nMinBufLength >= strlen(m_string))
  {
    char *p = (char*)realloc(m_string, nMinBufLength);
    if (p)
      m_string = p;
  }
  return m_string;
}

void XString::ReleaseBuffer(size_t nNewLength)
{
  if (nNewLength != strlen(m_string)+1)
  {
    char *p = (char*)realloc(m_string, nNewLength+1);
    m_string[nNewLength] = 0;
    if (p)
      m_string = p;
  }
}

const char * XString::GetBufferSetLength(size_t nNewLength)
{
  if (nNewLength != strlen(m_string)+1)
  {
    char *p = (char*)realloc(m_string, nNewLength+1);
    m_string[nNewLength] = 0;
    if (p)
      m_string = p;
  }
  return m_string;
}

void XString::FreeExtra()
{
}


const char * XString::LockBuffer()
{
    return m_string;
}

void XString::UnlockBuffer()
{
}


int XString::GetAllocLength() const
{
    return strlen(m_string);
}


char operator==(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) == 0);
}

char operator==(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) == 0);
}

char operator==(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) == 0);
}

char operator!=(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) != 0);
}

char operator!=(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) != 0);
}

char operator!=(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) != 0);
}

char operator<(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) < 0);
}

char operator<(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) < 0);
}

char operator<(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) < 0);
}

char operator>(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) > 0);
}

char operator>(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) > 0);
}

char operator>(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) > 0);
}

char operator<=(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) <= 0);
}

char operator<=(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) <= 0);
}

char operator<=(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) <= 0);
}

char operator>=(const XString& s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) >= 0);
}

char operator>=(const XString& s1, const char * s2)
{
    return (strcmp((const char *)s1, (const char *)s2) >= 0);
}

char operator>=(const char * s1, const XString& s2)
{
    return (strcmp((const char *)s1, (const char *)s2) >= 0);
}

char operator*=(const XString& s1, const XString& s2)
{
    return (stricmp((const char *)s1, (const char *)s2) == 0);
}

char operator*=(const XString& s1, const char * s2)
{
    return (stricmp((const char *)s1, (const char *)s2) == 0);
}

char operator*=(const char * s1, const XString& s2)
{
    return (stricmp((const char *)s1, (const char *)s2) == 0);
}

char operator/=(const XString& s1, const XString& s2)
{
    return (stricmp((const char *)s1, (const char *)s2) != 0);
}

char operator/=(const XString& s1, const char * s2)
{
    return (stricmp((const char *)s1, (const char *)s2) != 0);
}

char operator/=(const char * s1, const XString& s2)
{
    return (stricmp((const char *)s1, (const char *)s2) != 0);
}


#ifdef TEST_XSTRING

#define CCP(s) ((const char*)(s))

void t1(XString s)
{
  TRACE("t1(%p=\"%s\")", CCP(s), CCP(s));
}

int main(int argc, char **argv)
{
  XString s1 = "s1";
  XString s2 = "s2";
  XString s3 = "s3";
  short ii = 123;
  t1(s1);
  t1(s1|s2);
  t1(s1|"+"|1|"+");
  t1(s1|"+"|ii|"+");
  t1(XString(ii)|"+"|s1);
  printf("<%s> is%s blank\n", (const char*)s1, s1 ? " not" : "");
  s1 = "";
  printf("<%s> is%s blank\n", (const char*)s1, s1 ? " not" : "");
  s1 = NULL;
  printf("<%s> is%s blank\n", (const char*)s1, s1 ? " not" : "");
  s1 = "ThIS is a MIXed,caSE StRiNg";
  s2 = +s1;
  printf("+ '%s' = '%s'\n", (const char*)s1, (const char*)s2);
  s2 = -s1;
  printf("- '%s' = '%s'\n", (const char*)s1, (const char*)s2);
  s2 = ~s1;
  printf("~ '%s' = '%s'\n", (const char*)s1, (const char*)s2);
  s2 = *s1;
  printf("* '%s' = '%s'\n", (const char*)s1, (const char*)s2);
  s2 = PadRight("1234", 7);
  printf("PadRight(\"%s\", 7) = \"%s\"\n", "1234", CCP(s2));
  s2 = PadLeft("1234", 7);
  printf("PadLeft (\"%s\", 7) = \"%s\"\n", "1234", CCP(s2));
  s2 = Centre("1234", 7);
  printf("Centre  (\"%s\", 7) = \"%s\"\n", "1234", CCP(s2));
  s1 = "123abc1abc";
  printf("Find('a', 0) in \"%s\" = %d\n", CCP(s1), s1.Find('a', 0));
  printf("Find('a', 3) in \"%s\" = %d\n", CCP(s1), s1.Find('a', 3));
  printf("Find('a', 4) in \"%s\" = %d\n", CCP(s1), s1.Find('a', 4));
  printf("Find(\"abc\", 0) in \"%s\" = %d\n", CCP(s1), s1.Find("abc", 0));
  printf("Find(\"abc\", 3) in \"%s\" = %d\n", CCP(s1), s1.Find("abc", 3));
  printf("Find(\"abc\", 4) in \"%s\" = %d\n", CCP(s1), s1.Find("abc", 4));
  printf("Find(\"abc\", 9) in \"%s\" = %d\n", CCP(s1), s1.Find("abc", 9));
  return 0;
}

#endif
